# Copyright 2021 Zilliz. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the 'License');
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an 'AS IS' BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import unittest

from towhee.trainer.models.vit.vit_configs import get_b16_config


class VitConfigsTest(unittest.TestCase):
    config = get_b16_config()

    def test_vit_config(self):
        self.assertEqual(self.config['dim'], 768)


if __name__ == '__main__':
    unittest.main()
